using System;
using System.Collections.Generic;
using System.ServiceModel;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Http.Features;
using Microsoft.AspNetCore.Rewrite;
using Microsoft.AspNetCore.StaticFiles;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.FileProviders;
using Microsoft.Extensions.Hosting;
using System.Text;
using Microsoft.IdentityModel.Tokens;
using System.Linq;
using Microsoft.AspNetCore.Mvc.Razor;

namespace MySystem
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;

        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddControllersWithViews();
            services.AddRouting(options =>
            {
                options.LowercaseUrls = true;
            });
            services.AddSingleton<IHttpContextAccessor, HttpContextAccessor>();
            // services.AddCors(option => option.AddPolicy("cors", policy => policy.AllowAnyHeader().AllowAnyMethod().AllowCredentials().SetIsOriginAllowed(_ => true)));
            services.AddMvc(options =>
            {
                options.EnableEndpointRouting = false;
                options.Filters.Add(typeof(GlobalExceptionsFilter));
            });
            //配置模版视图路径
            services.Configure<RazorViewEngineOptions>(options =>
            {
                options.ViewLocationExpanders.Add(new TemplateViewLocationExpander());
            });
            services.AddSession(options =>
            {
                // 设置 Session 过期时间
                options.IdleTimeout = TimeSpan.FromHours(1);
                options.Cookie.HttpOnly = true;
            });
            services.Configure<FormOptions>(x =>
            {
                x.MultipartBodyLengthLimit = 50 * 1024 * 1024;//不到300M
            });
            //生成密钥
            var symmetricKeyAsBase64 = Configuration["Setting:JwtSecret"];
            var keyByteArray = Encoding.ASCII.GetBytes(symmetricKeyAsBase64);
            var signingKey = new SymmetricSecurityKey(keyByteArray);
            //认证参数
            services.AddAuthentication("Bearer").AddJwtBearer(o =>
            {
                o.TokenValidationParameters = new TokenValidationParameters
                {
                    ValidateIssuerSigningKey = true,//是否验证签名,不验证的画可以篡改数据，不安全
                    IssuerSigningKey = signingKey,//解密的密钥
                    ValidateIssuer = true,//是否验证发行人，就是验证载荷中的Iss是否对应ValidIssuer参数
                    // ValidIssuer = Configuration["Setting:JwtIss"],//发行人
                    IssuerValidator = (m, n, z) =>
                    {
                        return n.Issuer;
                    },
                    ValidateAudience = true,//是否验证订阅人，就是验证载荷中的Aud是否对应ValidAudience参数
                    // ValidAudience = Configuration["Setting:JwtAud"],//订阅人
                    AudienceValidator = (m, n, z) =>
                    {
                        string check = RedisDbconn.Instance.Get<string>("utoken:" + n.Issuer);
                        return m != null && m.FirstOrDefault().Equals(check);
                    },
                    ValidateLifetime = true,//是否验证过期时间，过期了就拒绝访问
                    ClockSkew = TimeSpan.Zero,//这个是缓冲过期时间，也就是说，即使我们配置了过期时间，这里也要考虑进去，过期时间+缓冲，默认好像是7分钟，你可以直接设置为0
                    RequireExpirationTime = true,
                };
            });

            MySystemLib.SystemPublicFuction.appcheck = "success";
            RedisDbconn.csredis = new CSRedis.CSRedisClient(Configuration["Setting:RedisConnStr"]);
            KxsRedisDbconn.csredis = new CSRedis.CSRedisClient(Configuration["Setting:KxsRedisConnStr"]);
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
                Library.ConfigurationManager.EnvironmentFlag = 1;
            }
            else
            {
                app.UseExceptionHandler("/Home/Error");
                app.UseHsts();
                Library.ConfigurationManager.EnvironmentFlag = 2;
            }
            Library.function.WritePage("/", "WebRootPath.txt", env.WebRootPath);

            app.UseStaticFiles();
            app.UseStaticFiles(new StaticFileOptions
            {
                FileProvider = new PhysicalFileProvider(AppContext.BaseDirectory + "/static"),
                RequestPath = "/static"
            });
            app.UseStaticFiles(new StaticFileOptions
            {
                ContentTypeProvider = new FileExtensionContentTypeProvider(new Dictionary<string, string>
                {
                      { ".apk", "application/vnd.android.package-archive" }
                })
            });

            app.UseCors("cors");
            app.UseAuthentication();
            app.UseRouting();
            app.UseAuthorization();
            app.UseSession();

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllerRoute(
                    name: "default",
                    pattern: "{controller=Home}/{action=Index}/{Id?}");
            });

            initMainServer();
            if (Library.ConfigurationManager.EnvironmentFlag == 1)
            {
                TestHaoDaService.Instance.Start(); //好哒测试
                // MerchantStandardService.Instance.Start();
            }
            if (Library.ConfigurationManager.EnvironmentFlag == 2)
            {
                HaodaParam.GetBoxInfo(); //配置盒子参数缓存

                GetStoreNoHelper.Instance.Start(); //获取商户门店号、微信支付宝商户号
                MerchantConfirmService.Instance.Start(); //提交商户进件
                CheckWeChatSignService.Instance.Start(); //查询商户审核状态
                CheckWeChatBindService.Instance.Start(); //执行好哒微信绑定appid
                ProfitShareService.Instance.Start(); //分账状态监控队列，分账完成则提交返现
                HaoDaExtHelper.Instance.StartWeChat();
                HaoDaExtHelper.Instance.StartAlipay();
                HaoDaExtQueryHelper.Instance.StartWeChat();
                HaoDaExtQueryHelper.Instance.StartAlipay();
                HaoDaExtQueryAuthHelper.Instance.StartWeChat();
                HaoDaExtQueryAuthHelper.Instance.StartAlipay();
                HaoDaAuthQueryHelper.Instance.Start(); //查询实名认证状态
                HaoDaFeeHelper.Instance.Start(); //设置费率
                WeChatPayBackService.Instance.Start(); //支付回调
                WeChatPayBackService.Instance.StartProfitShare(); //分账队列
                WeChatPayBackService.Instance.StartSettleOrder(); //接收分账结算金额
                WeChatPayBackService.Instance.StartDivi(); //补分账
                ProfitHelper.Instance.StartListenTrade(); //返现队列-支付宝
                ProfitHelper.Instance.StartListenWxTrade(); //返现队列-微信
                ProfitHelper.Instance.StartSetDivi(); //设置订单当前返现金额
                ProfitBeforeHelper.Instance.StartListenProfit(); //每月分润
                ProfitAfterHelper.Instance.StartListenProfit(); //每月分润
                ProfitCheckHelper.Instance.Start(); //检查订单是否在队列里

                AlipayPayBackFeeService.Instance.Start();
                ActiveRewardService.Instance.StartAct();
                ActiveRewardService.Instance.StartOpenReward();
                ActiveRewardService.Instance.StartLeaderReward();
                ActiveRewardService.Instance.StartOperateReward();

                MerchantStandardService.Instance.Start(); //商户缴纳服务费次月活动交易额大于等于1W，奖励进件创客50元
                MerchantStandardService.Instance.StartThree(); //商户缴纳服务费次月起连续不间断三个月，每月活动交易额大于1W，奖励进件创客100元
                MerchantStandardService.Instance.StartActive();
                
                GetTencentAddressInfoService.Instance.Start(); // 获取腾讯地图地址

                OrderPushHelper.Instance.Start(); //给商户单独推送订单信息


                // MerchantConfirmService.Instance.StartDo(11002);
            }
        }

        //初始化数据结构
        private void initMainServer()
        {
            Dictionary<string, Dictionary<string, string>> tables = new Dictionary<string, Dictionary<string, string>>();
            string connstr = Configuration["Setting:SqlConnStr"];
            System.Data.DataTable tablecollection = Library.CustomerSqlConn.dtable("select DISTINCT TABLE_NAME from information_schema.columns where table_schema = 'QrCodePlateMainServer'", connstr);
            foreach (System.Data.DataRow subtable in tablecollection.Rows)
            {
                Dictionary<string, string> Columns = new Dictionary<string, string>();
                System.Data.DataTable columncollection = Library.CustomerSqlConn.dtable("select COLUMN_NAME,DATA_TYPE from information_schema.columns where table_schema = 'QrCodePlateMainServer' and TABLE_NAME='" + subtable["TABLE_NAME"].ToString() + "'", connstr);
                foreach (System.Data.DataRow column in columncollection.Rows)
                {
                    string datatype = column["DATA_TYPE"].ToString();
                    if (datatype == "decimal")
                    {
                        datatype = "numeric";
                    }
                    Columns.Add(column["COLUMN_NAME"].ToString(), datatype);
                }
                tables.Add(subtable["TABLE_NAME"].ToString(), Columns);
            }
            AppConfig.Base.mainTables = tables;
        }
    }
}